* This macro calculates 
* 1) the PCI2 and 
* 2) generates a simulated PCI2 with associated statistics (e.g., mean, standard deviation)
* For a general discussion of PCI2 see Vaske, Beaman, Barreto, and Shelby (2009)
* This program allows for:
*          Different scale widths (i.e., 2, 3, 4, 5, 6, 7, 8, and 9-point)
*          Up to four Subsample / segmenting variables (see example below) can be used
*          Three different distance functions: 
*                  D1 - used with bipolar scales - excludes 0 "neutral" in the calculation of distance
*                         (example of bipolar scale:  -4  -3  -2  -1  0  +1  +2  +3  +4)
*                  D2 - used with bipolar scales - includes 0 "neutral" in the calculation of distance
*                  D3 - used with unipolar scales (e.g., Not at all important to extremely important)
*                         (Note - for unipolar scales the lowest value should be 0)
*          Different powers of distance (power should be > 0 and < 5)
*                  Power = 1 = unsquared powers of differences (recommended- see paper cited)
*                  Power = 2 = squared powers of differences 
*                  Powers greater than 1 account for non-linear perceptions of distance
*
* All files should be stored in the directory C:\PCI
* If this directory does not exist you will need to create it
* If you are using a network or jump drive (e.g., E:) you will need to change C:\PCI to X:\PCI throughout the macro (where X is the letter of the drive)
*
* To run the progam
*      1) highlight from DEFINE (line 30) to !ENDDEFINE (line 284)
*      2) click on RUN button 
*      3) Then follow the instructions / example below the macro

* SET MPRINT YES.

DEFINE PCI2  ( INFILE=!ENCLOSE ('|','|')
   /OUTFILE=!ENCLOSE ('|','|')
   /PCIOUT= !ENCLOSE ('|','|')
   /WORKDIR=!ENCLOSE ('|','|')
   /SubSample1=!TOKENS (1)
   /SubSample2=!TOKENS (1)
   /SubSample3=!TOKENS (1)
   /SubSample4=!TOKENS (1)
   /VAR= !TOKENS (1) 
   /Distance_function= !TOKENS (1)
   /Power_of_distance= !ENCLOSE ('|','|')
   /Scale_width= !TOKENS (1) ).

GET  FILE=!INFILE 
  /KEEP !SubSample1 !SubSample2 !SubSample3 !SubSample4 !VAR.

IF (!QUOTE(!Distance_function) NE 'D3')  #center=TRUNC((!Scale_width+1)/2).
IF (!QUOTE(!Distance_function) NE 'D3')  #V= !VAR +#center.
IF (!QUOTE(!Distance_function) = 'D3')    #V= !VAR + 1.
VECTOR TT(!Scale_width).
IF (#V>0) TT(#V)=1.

COMPUTE NoSubSample=1.
Formats NoSubSample (F1.0).

SAVE OUTFILEFILE='C:\PCI\Temp0.sav'.

SORT CASES BY NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4 .
AGGREGATE
  /OUTFILE=!QUOTE(!CONCAT(!WORKDIR, Prelim_Calc.sav)) 
  /PRESORTED
  /BREAK= NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4
  /TTSUM1=SUM(TT1) 
  /TTSUM2=SUM(TT2) 
  /TTSUM3=SUM(TT3) 
!IF (!Scale_width !GE 4) !THEN   !blanks(2)/TTSUM4=SUM(TT4)    !IFEND
!IF (!Scale_width !GE 5) !THEN   !blanks(2)/TTSUM5=SUM(TT5)    !IFEND
!IF (!Scale_width !GE 6) !THEN   !blanks(2)/TTSUM6=SUM(TT6)    !IFEND
!IF (!Scale_width !GE 7) !THEN   !blanks(2)/TTSUM7=SUM(TT7)    !IFEND
!IF (!Scale_width !GE 8) !THEN   !blanks(2)/TTSUM8=SUM(TT8)    !IFEND
!IF (!Scale_width !GE 9) !THEN   !blanks(2)/TTSUM9=SUM(TT9)    !IFEND 
   .
GET  FILE=!QUOTE(!CONCAT(!WORKDIR, Prelim_Calc.sav)).

FORMATS TTSUM1 TO !CONCAT(TTSUM,!Scale_width) (F5.0).
RECODE TTSUM1 TO !CONCAT(TTSUM,!Scale_width) (SYSMIS = 0).
EXECUTE.

SAVE TRANSLATE OUTFILE=!OUTFILE
     /REPLACE
     /TYPE=CSV
     /KEEP NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4  TTSUM1 TO !CONCAT(TTSUM,!Scale_width).
     
INPUT PROGRAM.
DATA LIST FILE=!OUTFILE FREE/ NoSubSample 
        !IF (!SubSample1 !NE !NULL) !THEN !CONCAT(#,!SubSample1) !IFEND 
        !IF (!SubSample2 !NE !NULL) !THEN !CONCAT(#,!SubSample2) !IFEND   
        !IF (!SubSample3 !NE !NULL) !THEN !CONCAT(#,!SubSample3) !IFEND 
        !IF (!SubSample4 !NE !NULL) !THEN !CONCAT(#,!SubSample4) !IFEND 
        #F1 TO !CONCAT(#F,!Scale_width).
COMPUTE #center=TRUNC((!Scale_width+1)/2).
COMPUTE F1=#F1.
COMPUTE !CONCAT(F,!Scale_width)=!CONCAT(#F,!Scale_width).
COMPUTE FF1=#F1.
COMPUTE !CONCAT(FF,!Scale_width)=!CONCAT(#F,!Scale_width).

* Establish records to output for simulated PCI.
* This simulation generates 400 records. 
* The default (n = 400) can be changed to whatever sample size is desired
* (e.g., change COMPUTE=#RPT=400 to COMPUTE=#RPT=1000)

COMPUTE #RPT=400.

* Establish vector for frequencies.

VECTOR #F=#F1 TO !CONCAT(#F,!Scale_width).

* Proceed with computation of actual PCI for an input distribution (Scale_width = odd).
* For actual PCI set Flag as 0 (to recognize it in output and keep it out of computation of mean and std using aggregate).

COMPUTE FLAG=0.

* This distance doe not include zero (i.e., 0) so distance from -1 to 1 is 1 (center is left out).
* Given definition, max distance is just Scale_width-2.

IF (!QUOTE(!Distance_function)='D1')  #MAXDIST=!Scale_width-2.
IF (!QUOTE(!Distance_function)='D2')  #MAXDIST=!Scale_width-1.
IF (!QUOTE(!Distance_function)='D3')  #MAXDIST=!Scale_width-1.

IF (!QUOTE(!Distance_function)='D1')  #SUBTRACT=1.
IF (!QUOTE(!Distance_function)='D2')  #SUBTRACT=0.
IF (!QUOTE(!Distance_function)='D3')  #SUBTRACT=0.

COMPUTE MAXDIST=#MAXDIST.
COMPUTE SUBTRACT=#SUBTRACT.

* Number of respondents is sum of #Fs.

COMPUTE #Respondents=SUM(#F1 TO !CONCAT(#F,!Scale_width)).
COMPUTE XRespondents=#Respondents.

* Determine maximum total of distance, #TOTDISMX

IF  (MOD(#Respondents,2)=0) #TOTDISMX=2*TRUNC(#Respondents/2)**2*#MAXDIST**!Power_of_distance.
IF  (MOD(#Respondents,2)=1) #TOTDISMX=2*TRUNC((#Respondents+1)/2)*TRUNC((#Respondents-1)/2)*#MAXDIST**!Power_of_distance.
COMPUTE TOTDISMX=#TOTDISMX.
COMPUTE #DISTSUM =0.

* Set first/outer loop upper limit.

COMPUTE #LUP=!Scale_width-1.
LOOP #L= 1 TO #LUP.
COMPUTE #IJ=#L+1.
LOOP #J= #IJ TO !Scale_width.
IF (!QUOTE(!Distance_function) NE 'D3' AND #L<#center AND #J>#center) #DISTSUM=#DISTSUM+2*#F(#L)*#F(#J)*(#J-#L-#SUBTRACT)**!Power_of_distance.
IF (!QUOTE(!Distance_function)='D3') #DISTSUM=#DISTSUM+2*#F(#L)*#F(#J)*(#J-#L)**!Power_of_distance.

END LOOP.
END LOOP.

* Distance has been totaled so PCI can be computed for actual data.

COMPUTE PCI=#DISTSUM/#TotDisMx.
COMPUTE NOSubSample=1.
 !IF (!SubSample1 !NE !NULL) !THEN COMPUTE !SubSample1= !CONCAT(#, !SubSample1). !IFEND
 
 !IF (!SubSample2 !NE !NULL) !THEN COMPUTE !SubSample2= !CONCAT(#, !SubSample2). !IFEND
 
 !IF (!SubSample3 !NE !NULL) !THEN COMPUTE !SubSample3= !CONCAT(#, !SubSample3). !IFEND 

 !IF (!SubSample4 !NE !NULL) !THEN COMPUTE !SubSample4= !CONCAT(#, !SubSample4). !IFEND 

END CASE.

* Generate PCI for #RPT quasi random samples based on frequencies.
* Set a vector to store new frequencies.

VECTOR #FF(!Scale_width).
LOOP FLAG=1 TO #RPT.

* Generate new frequencies based on the binomial.
* Set up loop to get #FF values.

LOOP #i=1 TO !Scale_width.

* For binomial get probability as #F(#i)/#Respondents.

if (#i=1) f1=#f(#i).
if (#i=!Scale_width) !CONCAT(F,!Scale_width)=#f(#i).
COMPUTE #PROB=0.
IF (#F(#i)>0)  #PROB=#F(#i)/#Respondents.
if (#i=1) PROB1=#PROB.
if (#i=!Scale_width) !CONCAT(PROB,!Scale_width)=#PROB.
COMPUTE #FF(#i)=RV.BINOM(#Respondents,#PROB).
if (#i=1) ff1=#ff(#i).
if (#i=!Scale_width) !CONCAT(FF,!Scale_width)=#ff(#i).
END LOOP.
COMPUTE #XRespondents=SUM(#FF1 to !CONCAT(#FF,!Scale_width)).
COMPUTE xRespondents=#xRespondents.

* Determine maximum total of distance, #TotDisMx.

IF  (MOD(#XRespondents,2)=0) #TotDisMx=2*TRUNC(#XRespondents/2)**2*#MAXDIST**!Power_of_distance.
IF  (MOD(#XRespondents,2)=1) #TotDisMx=2*TRUNC((#XRespondents+1)/2)*TRUNC((#XRespondents-1)/2)*#MAXDIST**!Power_of_distance.
COMPUTE TOTDISMX=#totdismx.
COMPUTE #DISTSUM=0.
COMPUTE MAX=0.
LOOP #L= 1 TO 4.
COMPUTE #IJ=#L+1.
LOOP #J= #IJ TO !Scale_width.
IF (!QUOTE(!Distance_function) NE 'D3' AND #L<#center AND #J>#center) #DISTSUM=#DISTSUM+2*#FF(#L)*#FF(#J)*(#J-#L-#SUBTRACT)**!Power_of_distance.
IF (!QUOTE(!Distance_function) = 'D3' ) #DISTSUM=#DISTSUM+2*#FF(#L)*#FF(#J)*(#J-#L-#SUBTRACT)**!Power_of_distance.
IF (#DISTSUM>MAX) MAX=#DISTSUM.
END LOOP.
END LOOP. 

COMPUTE PCI=#DISTSUM/#TotDisMx.
COMPUTE NOSubSample=1.
 !IF (!SubSample1 !NE !NULL) !THEN COMPUTE !SubSample1= !CONCAT(#, !SubSample1). !IFEND 
  
 !IF (!SubSample2 !NE !NULL) !THEN COMPUTE !SubSample2= !CONCAT(#, !SubSample2). !IFEND  
 
 !IF (!SubSample3 !NE !NULL) !THEN COMPUTE !SubSample3= !CONCAT(#, !SubSample3). !IFEND  
 
 !IF (!SubSample4 !NE !NULL) !THEN COMPUTE !SubSample4= !CONCAT(#, !SubSample4). !IFEND    
END CASE.
END LOOP.
END INPUT PROGRAM.


SAVE OUTFILE=!PCIOUT.

SORT CASES BY NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4 .

GET FILE =!PCIOUT
                 /KEEP NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4 PCI FLAG.
FILTER OFF.
USE ALL.
SELECT IF (FLAG ~= 0).
EXECUTE.

SPLIT FILE SEPARATE BY NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4.

SAVE OUTFILE=!QUOTE(!CONCAT(!WORKDIR, Sim_PCI.sav)). 
SPLIT FILE OFF.

AGGREGATE
  /OUTFILE=!QUOTE(!CONCAT(!WORKDIR, Sim_PCI_Mean_Std.sav ))
  /PRESORTED
  /BREAK= NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4
  /PCI_Sim_Mean=MEAN(PCI) 
  /PCI_Sim_Std=SD(PCI).

GET FILE=!QUOTE(!CONCAT(!WORKDIR, Sim_PCI_Mean_Std.sav)).

STRING PCIvar (A25).
COMPUTE PCIvar=!QUOTE (!VAR).
COMPUTE !CONCAT(PCI_Sim_Std,!Scale_width,!Distance_function,P,!Power_of_distance)=PCI_Sim_Std.
COMPUTE !CONCAT(PCIMN,!Scale_width,!Distance_function,P,!Power_of_distance)=PCI_Sim_Mean.

SAVE OUTFILEFILE='C:\PCI\Temp2.sav'.

GET FILE =!PCIOUT
                 /KEEP NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4 PCI XRespondents FLAG.
COMPUTE Sample=XRespondents.
Formats Sample (F8.0).

COMPUTE !CONCAT(PCI,!Scale_width,!Distance_function,P,!Power_of_distance)=PCI.
FILTER OFF.
USE ALL.
SELECT IF (FLAG = 0).
Compute PCI_Actual=PCI.
EXECUTE.

SAVE OUTFILE='C:\PCI\Temp1.sav'
          /DROP FLAG XRespondents PCI.
GET FILE='C:\PCI\Temp1.sav'.
MATCH FILES /FILE=*
           /FILE='C:\PCI\Temp2.sav'
           /BY NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4.
SAVE OUTFILE='C:\PCI\PCI_Actual_Simulated.sav'.
GET FILE='C:\PCI\PCI_Actual_Simulated.sav'.

LIST VARIABLES =PCIvar Sample !SubSample1 !SubSample2 !SubSample3 !SubSample4
                            PCI_Actual PCI_Sim_Mean PCI_Sim_Std.

* GET !QUOTE(!CONCAT(!WORKDIR, Sim_PCI.sav )) .
* SPLIT FILE SEPARATE BY NoSubSample !SubSample1 !SubSample2 !SubSample3 !SubSample4.

* DESCRIPTIVES VARIABLES=PCI 
*     /STATISTICS = SKEWNESS KURTOSIS.


EXECUTE.
!ENDDEFINE.

* The following generates PCI2 and simulation statistics
* The infile is your original data file. 
* The file name should not contain any spaces
* The Subsample (or segmenting) variables might be:
*       SEX (i.e., male, female)
*       EDUCATION (e.g., Less than college education, Bachelors or higher education)
* In this Galapagos example, the Subsample variable is "Island"
* There were 3 islands included in the sample
*  1) San Cristobal
*  2) Santa Cruz
*  3) Isabela
* The program allows for up to 4 subsample variables
* If you do not have any Subsample variables, delete the "SubSample" line
* If you have more than 1 Subsample variable, add more "SubSample" lines (up to 4 lines)
* 
* The Scale_width=4 statement should be changed to the scale width being used (i.e., 2, 3, 4, 5, 6, 7, 8 or 9)
* 
* Three distance_functions are available:  
*      D1 - used with bipolar scales - excludes 0 "neutral" in the calculation of distance
*             (example of bipolar scale:  -4  -3  -2  -1  0  +1  +2  +3  +4)
*      D2 - used with bipolar scales - includes 0 "neutral" in the calculation of distance
*      D3 - used with unipolar scales (e.g., Not at all important to extremely important)
*            (Note - for unipolar scales the lowest value should be 0)
*  The PCI variable (normfish) is a unipolar scale, so D3 is used in this example
*
* The power function is currently set to 1 (i.e, Power_of_distance=1)

* Example variables from Galapagos data
* Norms for Resource - normfish nozone nomethod notlobse notcucse sharkhar 
* Norms for Human Behavior - normbeha verbthre phythrea destruct phyagres
* Subsample - Island

SET MXWARNS=0.

PCI2 infile=|'C:\PCI\Galapagos.sav'|          
        outfile=|'C:\PCI\Galapagos.CSV'|     
        PCIOUT=|'C:\PCI\Sim_Data.sav'| 
        WORKDIR=|C:\PCI\|
        SubSample1=ISLAND 
        Scale_width=4
        var=normfish
        Distance_function=D3 
        Power_of_distance=|1|.

* The final calculated PCI and simulated PCI (mean and standard deviation ) are in "PCI_Actual_Simulated.sav"
* Before running a second variable move all files that were generated to different directory

* To test for differences in two actual PCI values use the equation below
* where
*          PCI_a = First actual PCI2 value
*          PCI_b = Second actual PCI2 value
*          PCI_Sim_Std_a = Simulated standard deviation for first PCI
*          PCI_Sim_Std_b = Simulated standard deviation for second PCI
* These values for variables will need to be entered into a new SPSS data file

compute testsd=ABS(PCI_a-PCI_b)/sqrt(PCI_Sim_Std_a**2+PCI_Sim_Std_b**2).

Execute.








